"""
netdisco.util.job
~~~~~~~~~~~~~~~~~

Access to Netdisco job metadata.
"""

import os
import json
from dataclasses import dataclass, InitVar

ND2_JOB_METADATA = json.loads(os.environ['ND2_JOB_METADATA'])


@dataclass(frozen=True)
class Job:
    job: int
    action: str
    entered: str = ''
    started: str = ''
    finished: str = ''
    device: str = ''
    port: str = ''
    only_namespace: str = ''
    subaction: str = ''
    status: str = ''
    username: str = ''
    userip: str = ''
    log: str = ''
    device_key: str = ''
    job_priority: str = ''
    is_cancelled: bool = False
    is_offline: bool = False

    _current_phase: InitVar[str] = ''
    _last_namespace: InitVar[str] = ''
    _last_priority: InitVar[int] = 0
    _statuslist: InitVar[list] = []  # field(default_factory=list)

    def id(self) -> int:
        return self.job


job = Job(**ND2_JOB_METADATA)



"""
netdisco.util.perl
~~~~~~~~~~~~~~~~~~

This module provides a utility function marshal_for_perl() to marshall a Python worklet's
status and stash for passing back to Netdisco Perl-side.
"""

import json
import base64
# from netdisco.util.log import debug


def marshal_for_perl(c):
    retval = {}

    if c is not None:
        retval = {'status': c.status.status, 'log': c.status.log, 'vars': c.stash.store}

    jstr = json.dumps(retval, default=str)
    # debug('returning: ' + jstr)
    return base64.b64encode(jstr.encode('ascii')).decode('ascii')



"""
netdisco.util.worklet
~~~~~~~~~~~~~~~~~~~~~

This module provides a "context" god-object instance with convenience functions helpful
for writing Python worklets. The context manages stash/vars and status, as well as providing
access to job metadata. For convenience netdisco.util.log.debug() is also shared.
"""

from dataclasses import dataclass, field
import netdisco.util.log as log
import netdisco.util.job as job
import netdisco.util.stash as stash
from netdisco.util.status import Status


@dataclass
class Stash:
    store: dict = field(default_factory=dict)

    def get(self, key):
        return stash.stash(key) or self.store[key]

    def set(self, key, val):
        self.store[key] = val


@dataclass(frozen=True)
class Context:
    job: object = job.job
    stash: Stash = Stash()
    status: Status = Status()


debug = log.debug
context = Context()



"""
netdisco.util.config
~~~~~~~~~~~~~~~~~~~~

Access to Netdisco runtime configuration.
"""

import os
import json

ND2_CONFIGURATION = json.loads(os.environ['ND2_CONFIGURATION'])


def setting(name):
    return ND2_CONFIGURATION[name]



"""
netdisco.util.log
~~~~~~~~~~~~~~~~~

This module provides a utility function debug() to emit log lines at level
DEBUG (if Netdisco configuration enables that, with "log: debug"). Log
messages are prefixed with the current process ID.
"""

import logging
from os import getpid
from netdisco.util.config import setting

logging.basicConfig(
    format=('[' + str(getpid()) + '] %(message)s'), level=getattr(logging, setting('log').upper())
)
log = logging.getLogger(__name__)


def debug(message):
    log.debug(str(message))
